@@ -22,7 +22,6 @@ Readonly my $_60_DEGREES     => 60;
 Readonly my $MAX_ZOOM        => 15;
 Readonly my $EMPTY_LIST      => -1;
 my $device;
-my %old_idles;
 
 my ( $_100_PERCENT, $MAX_CONFIDENCE_DEFAULT, $MIN_CONFIDENCE_DEFAULT );
 
@@ -31,7 +30,7 @@ BEGIN {
     Readonly $MAX_CONFIDENCE_DEFAULT => 95;
     Readonly $MIN_CONFIDENCE_DEFAULT => 50;
 }
-our $VERSION = '2.11.0';
+our $VERSION = '2.12.4';
 
 use Glib::Object::Subclass GooCanvas2::Canvas::, signals => {
     'zoom-changed' => {
@@ -269,14 +268,14 @@ sub string2rgb {
 }
 
 sub set_text {    # FIXME: why is this called twice when running OCR from tools?
-    my ( $self, $page, $edit_callback, $idle ) = @_;
+    my ( $self, $page, $layer, $edit_callback, $idle, $finished_callback ) = @_;
     if ( not defined $idle ) {
         $idle = TRUE;
     }
-    if (%old_idles) {
-        while ( my ( $box, $source ) = each %old_idles ) {
+    if ( $self->{old_idles} ) {
+        while ( my ( $box, $source ) = each %{ $self->{old_idles} } ) {
             Glib::Source->remove($source);
-            delete $old_idles{$box};
+            delete $self->{old_idles}{$box};
         }
     }
     delete $self->{position_index};
@@ -292,23 +291,24 @@ sub set_text {    # FIXME: why is this called twice when running OCR from tools?
 
     # Attach the text to the canvas
     $self->{confidence_index} = Gscan2pdf::Canvas::ListIter->new();
-    my $tree = Gscan2pdf::Bboxtree->new( $page->{bboxtree} );
+    my $tree = Gscan2pdf::Bboxtree->new( $page->{$layer} );
     my $iter = $tree->get_bbox_iter;
     my $box  = $iter->();
     if ( not defined $box ) { return }
     my %options = (
-        iter            => $iter,
-        box             => $box,
-        parents         => [$root],
-        transformations => [ [ 0, 0, 0 ] ],
-        edit_callback   => $edit_callback,
-        idle            => $idle,
+        iter              => $iter,
+        box               => $box,
+        parents           => [$root],
+        transformations   => [ [ 0, 0, 0 ] ],
+        edit_callback     => $edit_callback,
+        idle              => $idle,
+        finished_callback => $finished_callback,
     );
     if ($idle) {
-        $old_idles{$box} = Glib::Idle->add(
+        $self->{old_idles}{$box} = Glib::Idle->add(
             sub {
                 $self->_boxed_text(%options);
-                delete $old_idles{$box};
+                delete $self->{old_idles}{$box};
                 return Glib::SOURCE_REMOVE;
             }
         );
@@ -482,8 +482,10 @@ sub add_box {
 
     my @transformation = ( 0, 0, 0 );
     if ( $parent->isa('Gscan2pdf::Canvas::Bbox') ) {
-        my $parent_box = $parent->get('bbox');
-        @transformation = ( 0, $parent_box->{x}, $parent_box->{y} );
+        my $parent_box       = $parent->get('bbox');
+        my $parent_textangle = $parent->get('textangle');
+        @transformation =
+          ( $parent_textangle, $parent_box->{x}, $parent_box->{y} );
     }
     my %options2 = (
         parent         => $parent,
@@ -576,21 +578,25 @@ sub _boxed_text {
 
     push @transformations, [ $textangle + $rotation, $x1, $y1 ];
     my $child = $options{iter}->();
-    if ( not defined $child ) { return }
+    if ( not defined $child ) {
+        if ( $options{finished_callback} ) { $options{finished_callback}->() }
+        return;
+    }
 
     my %options3 = (
-        box             => $child,
-        iter            => $options{iter},
-        parents         => \@parents,
-        transformations => \@transformations,
-        edit_callback   => $edit_callback,
-        idle            => $options{idle},
+        box               => $child,
+        iter              => $options{iter},
+        parents           => \@parents,
+        transformations   => \@transformations,
+        edit_callback     => $edit_callback,
+        idle              => $options{idle},
+        finished_callback => $options{finished_callback},
     );
     if ( $options{idle} ) {
-        $old_idles{$child} = Glib::Idle->add(
+        $self->{old_idles}{$child} = Glib::Idle->add(
             sub {
                 $self->_boxed_text(%options3);
-                delete $old_idles{$child};
+                delete $self->{old_idles}{$child};
                 return Glib::SOURCE_REMOVE;
             }
         );
